# -*- coding: utf-8 -*-

# THE ALEXA COLOR PICKING SKILL

#
# THIS STUFF SETS UP OUR BACKEND, AND GET US READY TO RUN
#
import logging
from ask_sdk_core.skill_builder import SkillBuilder
from ask_sdk_core.utils import is_request_type, is_intent_name
from ask_sdk_core.handler_input import HandlerInput
from ask_sdk_model import Response
from ask_sdk_model.ui import SimpleCard

SKILL_NAME = "My Color Session"
HELP_TEXT = ("Please tell me your favorite color. You can say "
             "my favorite color is red")
COLOR_SLOT_KEY = "COLOR"
COLOR_SLOT = "Color"

sb = SkillBuilder()

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

#
# THIS HANDLES WHAT HAPPENS WHEN THE SKILL IS FIRST STARTED UP
#
@sb.request_handler(can_handle_func=is_request_type("LaunchRequest"))
def launch_request_handler(handler_input):
    speech = "Welcome to the Alexa Skills Kit color session sample."

    handler_input.response_builder.speak(
        speech + " " + HELP_TEXT).ask(HELP_TEXT)
    return handler_input.response_builder.response

#
# THIS HANDLES WHAT HAPPENS WHEN THE USER ASKS FOR HELP OR INFORMATION 
# ON THE SKILL
#
@sb.request_handler(can_handle_func=is_intent_name("AMAZON.HelpIntent"))
def help_intent_handler(handler_input):
    handler_input.response_builder.speak(HELP_TEXT).ask(HELP_TEXT)
    return handler_input.response_builder.response

#
# THIS HANDLES WHAT HAPPENS WHEN THE USER WANTS TO STOP OR CANCEL
# AN INTENT
#
@sb.request_handler(
    can_handle_func=lambda handler_input:
        is_intent_name("AMAZON.CancelIntent")(handler_input) or
        is_intent_name("AMAZON.StopIntent")(handler_input))
def cancel_and_stop_intent_handler(handler_input):
    speech_text = "Goodbye!"
    return handler_input.response_builder.speak(speech_text).response

# 
# THIS HANDLES WHAT HAPPENS WHEN A USER WANTS TO EXIT THE SKILL
#
@sb.request_handler(can_handle_func=is_request_type("SessionEndedRequest"))
def session_ended_request_handler(handler_input):
    return handler_input.response_builder.response

#
# HANDLES WHAT HAPPENS WHEN A USER ASKS WHAT THEIR FAVORITE COLOR IS
# CHECKS IF WE ALREADY KNOW THEIR COLOR:
#   - IF WE DO, RESPOND WITH THE COLOR. 
#   - IF WE DON'T, TELL THEM SO.
#
@sb.request_handler(can_handle_func=is_intent_name("WhatsMyColorIntent"))
def whats_my_color_handler(handler_input):
    if COLOR_SLOT_KEY in handler_input.attributes_manager.session_attributes:
        fav_color = handler_input.attributes_manager.session_attributes[
            COLOR_SLOT_KEY]
        speech = "Your favorite color is {}. Goodbye!!".format(fav_color)
        handler_input.response_builder.set_should_end_session(True)
    else:
        speech = "I don't think I know your favorite color. " + HELP_TEXT
        handler_input.response_builder.ask(HELP_TEXT)

    handler_input.response_builder.speak(speech)
    return handler_input.response_builder.response

#
# HANDLES WHAT HAPPENS WHEN A USER SAYS WHAT THEIR FAVORITE COLOR IS
# CHECKS WHETHER THE USER HAS PROVIDED A COLOR
#   - IF SO, SAVE THEIR FAVRITE COLOR
#   - IF NOT, TELL THEM WE DON'T UNDERSTAND
#
@sb.request_handler(can_handle_func=is_intent_name("MyColorIsIntent"))
def my_color_handler(handler_input):
    slots = handler_input.request_envelope.request.intent.slots

    if COLOR_SLOT in slots:
        fav_color = slots[COLOR_SLOT].value
        handler_input.attributes_manager.session_attributes[
            COLOR_SLOT_KEY] = fav_color
        speech = ("Now I know that your favorite color is {}. "
                  "You can ask me your favorite color by saying, "
                  "what's my favorite color ?".format(fav_color))
        reprompt = ("You can ask me your favorite color by saying, "
                    "what's my favorite color ?")
    else:
        speech = "I'm not sure what your favorite color is, please try again"
        reprompt = ("I'm not sure what your favorite color is. "
                    "You can tell me your favorite color by saying, "
                    "my favorite color is red")

    handler_input.response_builder.speak(speech).ask(reprompt)
    return handler_input.response_builder.response

#
# HANDLES UNEXPECTED USER REQUESTS
#
@sb.request_handler(can_handle_func=is_intent_name("AMAZON.FallbackIntent"))
def fallback_handler(handler_input):
    speech = (
        "The {} skill can't help you with that.  "
        "You can tell me your favorite color by saying, "
        "my favorite color is red").format(SKILL_NAME)
    reprompt = ("You can tell me your favorite color by saying, "
                "my favorite color is red")
    handler_input.response_builder.speak(speech).ask(reprompt)
    return handler_input.response_builder.response

#
# THE CODE BELOW LOGS INFO ANY TIME A RESPONSE IS SENT OFF
#
def convert_speech_to_text(ssml_speech):
    s = SSMLStripper()
    s.feed(ssml_speech)
    return s.get_data()


@sb.global_response_interceptor()
def add_card(handler_input, response):
    response.card = SimpleCard(
        title=SKILL_NAME,
        content=convert_speech_to_text(response.output_speech.ssml))


@sb.global_response_interceptor()
def log_response(handler_input, response):
    # type: (HandlerInput, Response) -> None
    print("Alexa Response: {}\n".format(response))

#
# LOGS INFO ANY TIME AN INTENT COMES IN
#
@sb.global_request_interceptor()
def log_request(handler_input):
    print("Alexa Request: {}\n".format(handler_input.request_envelope.request))

# 
# THE STUFF BELOW CATCHES ERRORS, AND LOGS THEM
#
@sb.exception_handler(can_handle_func=lambda i, e: True)
def all_exception_handler(handler_input, exception):
    print("Encountered following exception: {}".format(exception))

    speech = "Sorry, there was some problem. Please try again!!"
    handler_input.response_builder.speak(speech).ask(speech)

    return handler_input.response_builder.response


######## Convert SSML to Card text ############
# This is for automatic conversion of ssml to text content on simple card
# You can create your own simple cards for each response, if this is not
# what you want to use.

from six import PY2
try:
    from HTMLParser import HTMLParser
except ImportError:
    from html.parser import HTMLParser


class SSMLStripper(HTMLParser):
    def __init__(self):
        self.reset()
        self.full_str_list = []
        if not PY2:
            self.strict = False
            self.convert_charrefs = True

    def handle_data(self, d):
        self.full_str_list.append(d)

    def get_data(self):
        return ''.join(self.full_str_list)

################################################


# Handler to be provided in lambda console.
lambda_handler = sb.lambda_handler()